% ---------------------------------------------------------------------
% Publication:	Portfolio Decisions and Brain Reactions 
% ---------------------------------------------------------------------
% Quantlet:		DSFM_fmri
% ---------------------------------------------------------------------
% Description:	DSFM_BS_YC estimates DSFM model for aINS right cluster 
%               with B-spline basis functions
%               the program requires standard spline Matlab functions
% ---------------------------------------------------------------------
% Keywords:	    DSFM, splines, fmri, factor, dimension reduction, basis
% ---------------------------------------------------------------------
% Functions:	spap2, spcol, spmak, augknt, bkbrk, brk2knt
% ---------------------------------------------------------------------
% Inputs:		Clust_aINS_R.mat
% ---------------------------------------------------------------------
% Author:		Piotr Majer, Song Song, Szymon Borak 20140309
% ---------------------------------------------------------------------



load Clust_aINS_R
[J1 J2 J3 T]=size(DataGridTime); 
% 
 grid1=1:J1;  
  grid2=1:J2;  

   grid3=1:J3;  


     
%%%%%%%%%%%%%%%%%%%%%%DSFM%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

datapoints = zeros(length(grid1)*length(grid2)*length(grid3)*T,2);

ind =  1;
for tt = 1:T
tt;
    for g1=grid1
        for g2 =grid2
            for g3 =grid3
                datapoints(ind,:)=[DataGridTime(g1,g2,g3,tt),tt];
                ind=ind+1;
            end
        end
    end
end




datagrid=zeros(J1*J2*J3,3);
ind=1;
for g1=grid1
        for g2 =grid2
            for g3 =grid3
                datagrid(ind,:)=[g1,g2,g3];
                ind=ind+1;
            end
        end
end
datagrid=datagrid(1:ind-1,:);    

T = length(unique(datapoints(:,2)));


L=1; 
k1 = 3;
k2 = 3;
k3 = 3;


numknots1=3; 
numknots2=3; 
numknots3=3; 

% see the boundaries of the 3 dimensions
mn1 = min(datagrid(:,1))-1; mx1 = max(datagrid(:,1))+1;  
mn2 = min(datagrid(:,2))-1; mx2 = max(datagrid(:,2))+1;  
mn3 = min(datagrid(:,3))-1; mx3 = max(datagrid(:,3))+1;  

% equally divide [min, max] into #knots numbers.
knots1 = sort(linspace(mn1,mx1,numknots1)) ;

knots1 = augknt(knots1,k1);
knots2 = sort(linspace(mn2,mx2,numknots2)) ;
knots2 = augknt(knots2,k2);
knots3 = sort(linspace(mn3,mx3,numknots3)) ;
knots3 = augknt(knots3,k3);

% stop either reach the tolerence error, or 5 steps.
eps = 0.001;
maxiter = 5;

% set starting zetas, 1st column constant, 2,,,L are standard normal r.v.
z = [normrnd(0,1,T,L)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data=datapoints; startingzeta=z; epsilon=eps;

sDSFMobj = 0;

% number of splines numknots+ 2k1-2-k1=numknots+k1-2,
lk1 = length(knots1)-k1;
lk2 = length(knots2)-k2;
lk3 = length(knots3)-k3;
% %coefficients of splines, creats L+1 structs/m, initialize m hat
for ll=1:L
    coefs(ll).m = zeros(lk1,lk2,lk3);
end


tpoints = unique(data(:,2));
%number of time points
T = length(tpoints);

% row and columns of z
[ro,co]=size(startingzeta);
if  ro~=T
    error('Wrong number of time points in statring betas')
end

% see the boundaries of the 3 dimensions, setting the parameters
min1 = min(datagrid(:,1)); max1 = max(datagrid(:,1));
min2 = min(datagrid(:,2)); max2 = max(datagrid(:,2));
min3 = min(datagrid(:,3)); max3 = max(datagrid(:,3));


dim1 = 4; dim2 =4; dim3=4; %it seems to be enough
grid1 = linspace(min1,max1,dim1);
grid2 = linspace(min2,max2,dim2);
grid3 = linspace(min3,max3,dim3);

z = startingzeta;

[DU1 ,IIND1,IND1]=unique(datagrid(:,1));

[DU2 ,IIND2,IND2]=unique(datagrid(:,2));

[DU3 ,IIND3,IND3]=unique(datagrid(:,3));

% the collocation matrix 
SMART1 = spcol(knots1,k1,DU1)' ;
SMART2 = spcol(knots2,k2,DU2)' ;
SMART3 = spcol(knots3,k3,DU3)' ;

I3 = length(DU3);
obsperday = ones(length(tpoints),1);
for tp = 1: length(tpoints)
    ind = find(data(:,2)==tpoints(tp));
    obsperday(tp)=length(ind);
end

obsperday=[0;cumsum(obsperday)];

explainedvariance = 2*epsilon;
maxloop = maxiter;
description.explainedvariance = 100;
explainedvarianceold = explainedvariance+10*epsilon; 
breakingrule = 2*epsilon;
riteration = data(:,1);
r1iteration = 1000*epsilon*data(:,1); 

meandata = mean(data(:,1));
denominator =  sum((data(:,1)-meandata).^2);
description.breakingrule = breakingrule;

 krongrid3=datagrid(:,3); %get rid of memory problem 
for kkk=2:T
krongrid3=[krongrid3; datagrid(:,3)];
end

inddataday = find(data(:,2)==tpoints(1) & krongrid3==DU3(1));
size(inddataday)

MyPSIt = zeros(lk1*lk2,length(inddataday));
for ij=1:length(inddataday)
    MyPSIt(:,ij) = reshape(SMART1(:,IND1(inddataday(ij)))*SMART2(:,IND2(inddataday(ij)))',lk1*lk2,1);
end

for i3=1:I3 
    inddatadaynew(:,i3) = find(datagrid(:,3)==DU3(i3));
end



% main loop
while (maxloop>0  &  breakingrule>epsilon )
    maxloop
    for i3=1:I3
        if i3 == 1  % 1st cut
            MyXX = zeros((L)*lk1*lk2,(L)*lk1*lk2);
        end
        MyY  =  zeros((L)*lk1*lk2,1);

        for tt=2:length(obsperday) 
            inddataday=inddatadaynew(:,i3)+ (tt-2)*length(datagrid(:,3));
            if i3 == 1 %
             
                MyXX = MyXX + kron(z(tt-1,:)'*z(tt-1,:),MyPSIt*MyPSIt');
            end
            MyY  = MyY+kron(z(tt-1,:)',MyPSIt * data(inddataday,1));

        end
        c = MyXXMyY; 
        CC(:,i3)=c; 
    end % i3
    indi=1;
    for ll=1:(L) 
        for j=1:lk2 
            for i = 1:lk1 
                c =  (SMART3*SMART3')(SMART3*CC(indi,:)');
                for kkk=1:lk3 
                    coefs(ll).m(i,j,kkk)=c(kkk);
                end
                indi=indi+1; 
            end
        end
    end

    % regress on the time series
    nominator=0;
    modeliv=0;
    zz = z;
    ind = find(data(:,2)==tpoints(1));
%     M0 NOT HERE
    M1L= zeros(length(ind),L);
    for yyy = 1:length(ind)% 1: 6877
     
        BBB1 = SMART1(:,IND1(ind(yyy)));
        BBB2 = SMART2(:,IND2(ind(yyy)));
        BBB3 = SMART3(:,IND3(ind(yyy)));
%         M0(yyy)= 0;
%         for k=1:lk3 % 1:5
%             % M*phi
%             M0(yyy)=M0(yyy)+BBB1'*coefs(1).m(:,:,k)*BBB2*BBB3(k);
%         end
        for ll = 1:(L)
            for k=1:lk3
                M1L(yyy,ll)=M1L(yyy,ll)+BBB1'*coefs(ll).m(:,:,k)*BBB2*BBB3(k);
            end
        end
    end

    for tdate = 1:T

        ind = find(data(:,2)==tpoints(tdate));
        Y = data(ind,1);
        ztemp = (M1L'*M1L)(M1L'*(Y));

        z(tdate,1:(L))=ztemp'; % store z
        riteration(ind)=M1L*ztemp;
        nominator = nominator + sum((M1L*ztemp-Y).^2);

    end
    
    breakingrule = mean((riteration - r1iteration).^2);
    r1iteration = riteration;

    explainedvarianceold = explainedvariance;
    explainedvariance = nominator/denominator;
    description.explainedvariance = [description.explainedvariance;explainedvariance];
    description.breakingrule=[description.breakingrule;breakingrule];

    maxloop=maxloop-1;
end

INTEGRAL1=spcol(knots1,k1,grid1); 
INTEGRAL2=spcol(knots2,k2,grid2); 
INTEGRAL3=spcol(knots3,k3,grid3);

INTEGRAL1 = INTEGRAL1'*INTEGRAL1*(grid1(2)-grid1(1));
INTEGRAL2 = INTEGRAL2'*INTEGRAL2*(grid2(2)-grid2(1));
INTEGRAL3 = INTEGRAL3'*INTEGRAL3*(grid3(2)-grid3(1));
matrint = ones(lk1*lk2*lk3,lk1*lk2*lk3);

'matrint in process'
matrint=kron(kron(INTEGRAL3,INTEGRAL2),INTEGRAL1);
'matrint done'

GAMMA = ones(L);%gamma = ones(L,1);

for i = 1:L
    Ahat(i,:)=reshape(coefs(i).m(:,:,:),1,lk1*lk2*lk3);
end
for i = 1:L
    for j = 1:L
        GAMMA(i,j)=sum(sum(Ahat(i,:)*Ahat(j,:)'.*matrint));
    end
end

znew = z;
znew(:,1:end)=z(:,1:end)*GAMMA^0.5;
Ahatnew = Ahat;
Ahatnew(1:end,:) = (GAMMA^-0.5)*Ahat(1:end,:);

% ordering with respect to variance
B = znew(:,1:end)'*znew(:,1:end);
[V,D] = eigs(B);

zhat = znew;
zhat(:,1:end) = zhat(:,1:end)*V;
Ahatort = Ahatnew;
Ahatort(1:end,:) = V'*Ahatnew(1:end,:);

zmean = mean(zhat);
zhat(:,1:end)=zhat(:,1:end)-ones(length(zhat(:,1)),1)*zmean(1:end);
Ahatort(1,:) = Ahatort(1,:) +  zmean(1:end)*Ahatort(1:end,:);

coefsort = coefs;
for i = 1:L
    coefsort(i).m(:,:,:)=reshape(Ahatort(i,:),[lk1,lk2,lk3]);
end

description.eps = epsilon;
description.iter = maxiter-maxloop;

sDSFMobj.ydata = data;

sDSFMobj.z = zhat;
sDSFMobj.mcoefs = coefsort;
sDSFMobj.knots1=knots1;
sDSFMobj.k1=k1;
sDSFMobj.knots2=knots2;
sDSFMobj.k2=k2;
sDSFMobj.knots3=knots3;
sDSFMobj.k3=k3;
sDSFMobj.description = description;% 
